/************************************************************
Synchronizes the searchbox with a search engine in firefox
Copyright (C) 2005  Angus McBride

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
************************************************************/

var searchengines = new Array();
var ssPrefs;
load();

function load() {
	ssPrefs = getprefs();	//Fill the global variable ssPrefs with the preferences

	searchengines = getEngineArray();	//Put all of the searchengines into the search engine array
	
	document.getElementById("appcontent").addEventListener("load",checkUrl,true);	


}


function getprefs() {
	//Get preferences object
	var allprefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefService);

	var ssprefs = allprefs.getBranch("extensions.searchsynchronizer.");
	return ssprefs;
}


function getEngineArray() {
	var se = new Array();
	
	se = get_sengines();
	se = get_defaults(ssPrefs,se);

	return se;
}


function get_sengines() {
	var num = ssPrefs.getIntPref("numengines");
	var prefix = "custom.";
	var i,name,test,find;
	var se = new Array();	

	for(i=0;i<num;i++) {
		name = ssPrefs.getCharPref(prefix+i+".name");
		test = ssPrefs.getCharPref(prefix+i+".test");
		find = ssPrefs.getCharPref(prefix+i+".find");

		test = new RegExp(test,"i");
		find = new RegExp(find,"i");

		se[i] = new sengine(name,test,find);
	}
	return se;
}



//Searchengine (javascript equivelant of a )class
function sengine(name,test,find) {
	this.name = name;
	this.regtest = test;
	this.find = find;

	this.findregex = function() {
		return this.find;
	}
	this.getname = function() {
		return this.name;
	}
}



function checkUrl() {

	//Get the url
	var url = new String(window._content.document.location);

	for(i=0;i<searchengines.length;i++) {
		if(searchengines[i].regtest.test(url)) 
			if(updateSearchBox(url,searchengines[i]))
				break;
	}
}

function updateSearchBox(url,se) {
	//Declare Variables
	var regex = se.findregex();

	//Execute the regex
	var result = regex.exec(url);
	var search = result[1];
	
	//If result not set, stop
	if(search=="") {regex.exec(); return false;}
	
	//Convert the result to plain english
	search = plain_english(search,se);

	//Wikipedia fix
	if(se.getname()=='wiki'&&search=='Main_Page') {regex.exec();return false;}

	//Update the searchbox
	try{
		document.getElementById("searchbar").mTextbox.value = search;
	}
	catch(e) {
		document.getElementById("searchbar").value=search;
	}
	//Fix bug which prevented the regex from working a second time
	regex.exec();
	
	//Tell anything watching the searchbar that there has been input
        var evt = document.createEvent("Events");
        evt.initEvent("oninput", true, true);
        document.getElementById("searchbar").dispatchEvent(evt);

	return true;
}

//Convert url encoded string to plain english
function plain_english(str,se) {
	str = str.replace(/\+/g,' ');
	str = decodeURIComponent(str); //This is better than unescape() because it supports unicode 
	
	if(se.getname()=='wiki') str=str.replace(/_/g,' ');
	if(se.getname()=='ebay1') str=str.replace(/-/g,' ');

	return str;
}





var ssPrefListener =
{
  register: function()
  {
    this._branch = ssPrefs;

    var pbi = this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver("", this, false);
  },

  observe: function(aSubject, aTopic, aData)
  {
    if(aTopic != "nsPref:changed") return;
    // aSubject is the nsIPrefBranch we're observing

	searchengines = getEngineArray();
  }
}
ssPrefListener.register();	//Watch for changes to the preferences